package de.hft.stuttgart.citydoctor2.datastructure;

import de.hft.stuttgart.citydoctor2.check.Check;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.CityGmlUtils;
import de.hft.stuttgart.citydoctor2.utils.CopyHandler;
import de.hft.stuttgart.citydoctor2.utils.Copyable;
import org.citygml4j.core.model.core.AbstractCityObject;
import org.citygml4j.core.util.geometry.GeometryFactory;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurface;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurfaceProperty;
import org.xmlobjects.gml.model.geometry.primitives.Solid;
import org.xmlobjects.gml.model.geometry.primitives.SolidProperty;

import java.io.Serial;

public class CityFurniture extends CityObject{

    @Serial
    private static final long serialVersionUID = 3325661061543962473L;

    private org.citygml4j.core.model.cityfurniture.CityFurniture cgmlCityFurniture;

    @Override
    public void accept(Check c) {
        super.accept(c);
        if (c.canExecute(this)) {
            c.check(this);
        }
    }


    @Override
    public AbstractCityObject getGmlObject() {
        return cgmlCityFurniture;
    }

    public void setGmlObject(org.citygml4j.core.model.cityfurniture.CityFurniture gmlCityFurniture) {
        cgmlCityFurniture = gmlCityFurniture;
    }

    @Override
    public void unsetGmlGeometries() {
        cgmlCityFurniture.setLod0MultiSurface(null);
        cgmlCityFurniture.setLod2MultiSurface(null);
        cgmlCityFurniture.setLod3MultiSurface(null);
        cgmlCityFurniture.setLod1Solid(null);
        cgmlCityFurniture.setLod2Solid(null);
        cgmlCityFurniture.setLod3Solid(null);
        cgmlCityFurniture.getDeprecatedProperties().setLod4Geometry(null);
    }

    @Override
    public void reCreateGeometries(GeometryFactory factory, ParserConfiguration config) {
        for (Geometry geom : getGeometries()) {
            if (geom.getType() == GeometryType.MULTI_SURFACE) {
                MultiSurface ms = CityGmlUtils.createMultiSurface(geom, factory, config);
                setMultiSurfaceAccordingToLod(geom, ms);
            } else {
                Solid solid = CityGmlUtils.createSolid(geom, factory, config);
                setSolidAccordingToLod(geom, solid);
            }
        }
    }

    private void setMultiSurfaceAccordingToLod(Geometry geom, MultiSurface ms) {
        switch (geom.getLod()) {
            case LOD0:
                cgmlCityFurniture.setLod0MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD2:
                cgmlCityFurniture.setLod2MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD3:
                cgmlCityFurniture.setLod3MultiSurface(new MultiSurfaceProperty(ms));
                break;
            default:
                throw new IllegalStateException("Cannot add " + geom.getLod() + " multi surface to buildings");
        }
    }

    private void setSolidAccordingToLod(Geometry geom, Solid solid) {
        switch (geom.getLod()) {
            case LOD1:
                cgmlCityFurniture.setLod1Solid(new SolidProperty(solid));
                break;
            case LOD2:
                cgmlCityFurniture.setLod2Solid(new SolidProperty(solid));
                break;
            case LOD3:
                cgmlCityFurniture.setLod3Solid(new SolidProperty(solid));
                break;
            default:
                throw new IllegalStateException("Cannot add " + geom.getLod() + " solid to buildings");
        }
    }

    @Override
    public FeatureType getFeatureType() {
        return FeatureType.CITY_FURNITURE;
    }

    @Override
    public Copyable createCopyInstance() {
        return new CityFurniture();
    }

    @Override
    public void fillValues(Copyable original, CopyHandler handler) {
        super.fillValues(original, handler);
        CityFurniture originalCF = (CityFurniture) original;
        cgmlCityFurniture = originalCF.cgmlCityFurniture;
    }
}
